/*
 * ThermoStatContainer.cpp
 *
 *  Created on: 12.06.2010
 *      Author: heber
 */

#include <cstring>

#include "ConfigFileBuffer.hpp"
#include "config.hpp"
#include "log.hpp"
#include "ThermoStatContainer.hpp"
#include "verbose.hpp"

/** Constructor for class ThermoStatContainer.
 *
 */
ThermoStatContainer::ThermoStatContainer() : Thermostat(4), ThermostatImplemented(NULL),
ThermostatNames(NULL), TempFrequency(2.5), alpha(0.), HooverMass(0.), TargetTemp(0.00095004455), ScaleTempStep(25)
{
  ThermostatImplemented = new int[MaxThermostats];
  ThermostatNames = new char *[MaxThermostats];
  for (int j=0;j<MaxThermostats;j++)
    ThermostatNames[j] = new char[12];

  strcpy(ThermostatNames[0],"None");
  ThermostatImplemented[0] = 1;
  strcpy(ThermostatNames[1],"Woodcock");
  ThermostatImplemented[1] = 1;
  strcpy(ThermostatNames[2],"Gaussian");
  ThermostatImplemented[2] = 1;
  strcpy(ThermostatNames[3],"Langevin");
  ThermostatImplemented[3] = 1;
  strcpy(ThermostatNames[4],"Berendsen");
  ThermostatImplemented[4] = 1;
  strcpy(ThermostatNames[5],"NoseHoover");
  ThermostatImplemented[5] = 1;
}

/** Destructor for Class ThermoStatContainer.
 *
 */
ThermoStatContainer::~ThermoStatContainer()
{
  delete[](ThermostatImplemented);
  for (int j=0;j<MaxThermostats;j++)
    delete[](ThermostatNames[j]);
  delete[](ThermostatNames);
}


/** Reading of Thermostat related values from parameter file.
 * \param *fb file buffer containing the config file
 */
void ThermoStatContainer::ParseThermostats(class ConfigFileBuffer * const fb)
{
  char * const thermo = new char[12];
  const int verbose = 0;

  // read desired Thermostat from file along with needed additional parameters
  if (ParseForParameter(verbose,fb,"Thermostat", 0, 1, 1, string_type, thermo, 1, optional)) {
    if (strcmp(thermo, ThermostatNames[0]) == 0) { // None
      if (ThermostatImplemented[0] == 1) {
        Thermostat = None;
      } else {
        DoLog(1) && (Log() << Verbose(1) << "Warning: " << ThermostatNames[0] << " thermostat not implemented, falling back to None." << endl);
        Thermostat = None;
      }
    } else if (strcmp(thermo, ThermostatNames[1]) == 0) { // Woodcock
      if (ThermostatImplemented[1] == 1) {
        Thermostat = Woodcock;
        ParseForParameter(verbose,fb,"Thermostat", 0, 2, 1, int_type, &ScaleTempStep, 1, critical); // read scaling frequency
      } else {
        DoLog(1) && (Log() << Verbose(1) << "Warning: " << ThermostatNames[0] << " thermostat not implemented, falling back to None." << endl);
        Thermostat = None;
      }
    } else if (strcmp(thermo, ThermostatNames[2]) == 0) { // Gaussian
      if (ThermostatImplemented[2] == 1) {
        Thermostat = Gaussian;
        ParseForParameter(verbose,fb,"Thermostat", 0, 2, 1, int_type, &ScaleTempStep, 1, critical); // read collision rate
      } else {
        DoLog(1) && (Log() << Verbose(1) << "Warning: " << ThermostatNames[0] << " thermostat not implemented, falling back to None." << endl);
        Thermostat = None;
      }
    } else if (strcmp(thermo, ThermostatNames[3]) == 0) { // Langevin
      if (ThermostatImplemented[3] == 1) {
        Thermostat = Langevin;
        ParseForParameter(verbose,fb,"Thermostat", 0, 2, 1, double_type, &TempFrequency, 1, critical); // read gamma
        if (ParseForParameter(verbose,fb,"Thermostat", 0, 3, 1, double_type, &alpha, 1, optional)) {
          DoLog(2) && (Log() << Verbose(2) << "Extended Stochastic Thermostat detected with interpolation coefficient " << alpha << "." << endl);
        } else {
          alpha = 1.;
        }
      } else {
        DoLog(1) && (Log() << Verbose(1) << "Warning: " << ThermostatNames[0] << " thermostat not implemented, falling back to None." << endl);
        Thermostat = None;
      }
    } else if (strcmp(thermo, ThermostatNames[4]) == 0) { // Berendsen
      if (ThermostatImplemented[4] == 1) {
        Thermostat = Berendsen;
        ParseForParameter(verbose,fb,"Thermostat", 0, 2, 1, double_type, &TempFrequency, 1, critical); // read \tau_T
      } else {
        DoLog(1) && (Log() << Verbose(1) << "Warning: " << ThermostatNames[0] << " thermostat not implemented, falling back to None." << endl);
        Thermostat = None;
      }
    } else if (strcmp(thermo, ThermostatNames[5]) == 0) { // Nose-Hoover
      if (ThermostatImplemented[5] == 1) {
        Thermostat = NoseHoover;
        ParseForParameter(verbose,fb,"Thermostat", 0, 2, 1, double_type, &HooverMass, 1, critical); // read Hoovermass
        alpha = 0.;
      } else {
        DoLog(1) && (Log() << Verbose(1) << "Warning: " << ThermostatNames[0] << " thermostat not implemented, falling back to None." << endl);
        Thermostat = None;
      }
    } else {
      DoLog(1) && (Log() << Verbose(1) << " Warning: thermostat name was not understood!" << endl);
      Thermostat = None;
    }
  } else {
    if ((TargetTemp != 0))
      DoLog(2) && (Log() << Verbose(2) <<  "No thermostat chosen despite finite temperature MD, falling back to None." << endl);
    Thermostat = None;
  }
  delete[](thermo);
};

